

# wg == weight of grain
# mt == mash thickness (qts/lb)
# t_f == desired temp (F)
# t_a == ambient temp (F)
# sh_grain == specific heat of grain
# ti_tun == thermal inertia of mash tun

proc calc_strike {wg mt t_f t_a sh_grain ti_tun} {
  set cf 2.087
   return [expr {($t_f + ($t_f - $t_a)*($sh_grain + $ti_tun/$wg)/(1.00*$cf*$mt))}]
}


proc calc_water {wg mt t_f t_i t_w sh_grain ti_tun} {
  set cf 2.087
  return [expr {$cf*($t_f - $t_i)/($t_w - $t_f)*($ti_tun/$wg + $sh_grain + $mt/$cf)}]
}


proc full_stepmash {} {
   global sh_grain ti_tun boil_temp ambient_temp mash_thick totalweight
   global steps

   toplevel .sm
   set font {Helvetica 14 bold}

   if {![info exists totalweight] || [streqx $totalweight "" 0]} {
      tk_messageBox -type ok -message "Enter grain data and CALC first"
      return
   }


   wm geometry .sm 800x480+0+0
   canvas .sm.c -background white -width 800 -height 230
   text .sm.t -background grey -foreground black -font {Courier 18 bold} \
      -height 10
   pack .sm.c .sm.t -side top -expand y -fill x
   button .sm.exit -text Dismiss -command {destroy .sm} -foreground white \
      -background black -font $font
   pack .sm.exit -side top -expand y -fill x

   set w .sm.c

   set yval(1) 46
   set yval(2) 76
   set yval(3) 106
   set yval(4) 136
   set yval(5) 166
   set yval(6) 196

   $w create text 54 $yval(1) -text "Weight of grain (lbs):  " -anchor w -font $font
   label $w.wtgr -width 10 -textvariable totalweight -font $font -background white
   $w create window 240 $yval(1) -window $w.wtgr -anchor w

   $w create text 400 $yval(1) -text "Ambient temp (F):  " -anchor w -font $font
   entry $w.ambt -width 10 -textvariable ambient_temp -font $font -background white
   $w create window 586 $yval(1) -window $w.ambt -anchor w


   $w create text 54 $yval(2) -text "Thickness (qts/lb) rest 1:  " -anchor w -font $font
   entry $w.mshth -width 10 -textvariable mash_thick -font $font -background white
   $w create window 240 $yval(2) -window $w.mshth -anchor w

   $w create text 400 $yval(2) -text "Spec. Heat of Grain:  " -anchor w -font $font
   entry $w.shgr -width 10 -textvariable sh_grain -font $font -background white
   $w create window 586 $yval(2) -window $w.shgr -anchor w

   $w create text 54 $yval(3) -text "Thermal inertia of mash tun:  " -anchor w -font $font
   entry $w.tun -width 10 -textvariable ti_tun -font $font -background white
   $w create window 265 $yval(3) -window $w.tun -anchor w
   $w create text 380 $yval(3) -text "(0 if pre-heated ; 0.7 for 10gal cooler)" \
      -anchor w -font $font

   $w create text 54 $yval(4) -text "Enter desired steps (F) for mash" \
      -anchor w -font $font
   entry $w.steps -width 20 -textvariable steps -font $font -background white
   $w create window 340 $yval(4) -window $w.steps -anchor w

   button $w.calc -text Calculate -font $font -background white \
      -foreground black -command { calc_stepmash }
   $w create window 586 $yval(4) -window $w.calc -anchor w

   $w create text 54 $yval(5) -text "Typical rest temperatures:" -anchor w -font $font
   $w create text 54 $yval(6) -text "   Acid rest:  95F (lowers pH)  //  Protein rest:  120F to 130F  //  Saccharification rest:  148F to 158F" -anchor w -font $font

   calc_si_stepmash
}

proc tputs {s} { .sm.t insert end "$s\n" }
proc tclear {} { .sm.t delete 0.0 end }

proc calc_si_stepmash {} {
   global sh_grain ti_tun boil_temp ambient_temp mash_thick totalweight
   global mashtemp stepmash texstepmash

   set steplist [list $mashtemp 0]
   set temp_step [list $mashtemp]
   set mash_thk [list $mash_thick]

   set strike_temp [calc_strike $totalweight [lindex $mash_thk 0] \
                   [lindex $temp_step 0] $ambient_temp $sh_grain $ti_tun]
  
   set i 0
   set stepidx 0
   lappend steplist 0 ;# last entry
   set done 0
   while {!$done} {
      incr stepidx
      set s [lindex $steplist $stepidx]
      if {$s == 0} {
         set done 1
      } else {
         lappend temp_step $s
      }

      set water_add [calc_water $totalweight [lindex $mash_thk $i] $s \
            [lindex $temp_step $i] $boil_temp $sh_grain $ti_tun]
      lappend mash_thk [expr {[lindex $mash_thk $i] + $water_add}]
      incr i
   }

#  tputs " Temp.(F)   Thickness(qt/lb)   H2O(qt)"

   set i 0
   set tmp1 [format %3.0f $strike_temp]
   set tmp2 [format %.2f 0.0]
   set tmp3 [format %.1f [expr {$totalweight*[lindex $mash_thk $i]}]]
#  tputs "  $tmp1            $tmp2            $tmp3"

   for {set i 0} {$i < [llength $temp_step]} {incr i} {
      set tmp1 [format %3.0f $strike_temp]
      set tmp2 [format %.2f [lindex $mash_thk $i]]
      set tmp3 [format %.1f [expr {$totalweight*[lindex $mash_thk $i]}]]

#     tputs "  $tmp1            $tmp2            $tmp3"
      set gal [format %.2f [expr {$tmp3/4.00}]]
      .c.stepmash configure -state normal
      set stepmash "Strike (for $tmp2 qt/lb, 1 rest @ $mashtemp):  $tmp3 qt ($gal gallons) @ $tmp1 F."
      set texstepmash "Strike (for $tmp2 qt/lb, 1 rest @ $mashtemp\\degf):  $tmp3 qt ($gal gallons) @ $tmp1\\degf."
   }
}

proc calc_stepmash {} {
   global sh_grain ti_tun boil_temp ambient_temp mash_thick totalweight
   global steps

   set steplist [split $steps " "]
   set t [lindex $steplist 0]
   set temp_step [list $t]
   set mash_thk [list $mash_thick]
   tclear

   set strike_temp [calc_strike $totalweight [lindex $mash_thk 0] \
                   [lindex $temp_step 0] $ambient_temp $sh_grain $ti_tun]
  
   set i 0
   set stepidx 0
   lappend steplist 0 ;# last entry
   set done 0
   while {!$done} {
      incr stepidx
      set s [lindex $steplist $stepidx]
      if {$s == 0} {
         set done 1
      } else {
         lappend temp_step $s
      }

      set water_add [calc_water $totalweight [lindex $mash_thk $i] $s \
            [lindex $temp_step $i] $boil_temp $sh_grain $ti_tun]
      lappend mash_thk [expr {[lindex $mash_thk $i] + $water_add}]
      incr i
   }

   tputs ""
   set i 0
   set tmp1 [format %3.0f $strike_temp]
   set tmp2 [format %.2f 0.0]
   set tmp3 [format %.1f [expr {$totalweight*[lindex $mash_thk $i]}]]
   set tmp3gal [format %.2f [expr {1.0000*$tmp3/4}]]

   set step1 [format %3.0f [lindex $temp_step 0]]
   set thick1 [format %.2f [lindex $mash_thk 0]]
   set volqts $tmp3

   tputs "Rest #1:  $tmp3 qts ($tmp3gal gal) at  ${tmp1}F for ${step1}F at $thick1 qts/lb"

   for {set i 1} {$i < [llength $temp_step]} {incr i} {
      set tmp1 [format %3.0f [lindex $temp_step $i]]
      set tmp2 [format %.2f [lindex $mash_thk $i]]
      set tmp3 [format %.1f [expr {$totalweight*[lindex $mash_thk $i]}]]
      set add3 [format %.1f [expr {1.0000 * ($tmp3 - $volqts)}]]
      set add3gal [format %.1f [expr {1.0000 * ($add3/4)}]]

      tputs ""
      tputs "Rest #[expr {$i + 1}]:  Add $add3 qts ($add3gal gal) at 210F for ${tmp1}F"
      set tot3gal [format %.1f [expr {1.0000 * ($tmp3/4)}]]
      tputs "Total volume now at $tmp3 qts ($tot3gal gal) at $tmp2 qts/lb"
   }
}

# source lib/tkcon_wrap.tcl

 
